<?php

function permission_watchdog_admin_settings() {
  $form = array();

  $roles = array('all' => t('All roles')) + user_roles();

  $form['permsission_watchdog_roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Track changes to permissions on the following roles.'),
    '#options' => $roles,
    '#default_value' => variable_get('permsission_watchdog_roles', array()),
  );

  return system_settings_form($form);
}

function permission_watchdog_report($form, $form_state) {
  $form = array();
  $headers = array(t('User'), t('Role'), t('Timestamp'), t('Changes'));
  $permission_labels = array();
  foreach (module_implements('perm') as $module) {
    if ($permissions = module_invoke($module, 'perm')) {
      $path = drupal_get_path('module', $module) . '/' . $module . '.info';
      $info = drupal_parse_info_file($path);
      $labels = array();

      foreach($permissions as $k => $permission) {
        $labels[$permission] = $info['name'];
      }
      $permission_labels = array_merge($permission_labels, $labels);
    }
  }

  $query = "SELECT `change_id`, `uid`, `rid`, `stamp`, `data` FROM {permission_watchdog}";
  $count_query = "SELECT COUNT(`change_id`) FROM {permission_watchdog}";

  $session = ($_SESSION['permission_watchdog_filter']) ? $_SESSION['permission_watchdog_filter'] : array();

  if(isset($session['roles'])) {
    $query = sprintf($query . "WHERE `rid` IN (%s)", implode(", ", $session['roles']));
    $count_query = sprintf($query . " WHERE `rid` IN (%s)", implode(', ', $session['roles']));
  }

  $result = pager_query($query, 30, 0, $count_query);

  $rows = array();

  while($row = db_fetch_object($result)) {
    $changes = unserialize($row->data);
    $role = db_fetch_object(db_query('SELECT r.name FROM {role} r LEFT JOIN {permission} p ON r.rid = p.rid WHERE r.rid = %d', $row->rid));
    $items = array();
    foreach($changes as $permission => $change) {
      $change = ($change) ? t('Added') : t('Removed');
      $items[] = t('%change <strong>@module:</strong> !permission', array('%change' => $change,
                    '@module' => $permission_labels[$permission],
                    '!permission' => $permission));
    }

    $rows[] = array(theme('username', user_load($row->uid)),
                    $role->name,
                    format_date($row->stamp),
                    theme('item_list', $items));
  }

  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => 'Filters',
  );

  $form['filters']['roles'] = array(
    '#type' => 'select',
    '#multiple' => 'multple',
    '#title' => t('Roles'),
    '#options' => user_roles(),
    '#default_value' => $session['roles'],
  );

  $form['filters']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
  );

  $form['table'] = array('#value' => theme('table', $headers, $rows));

  $form['pager'] = array('#value' => theme('pager'));

  return $form;
}

function permission_watchdog_report_submit($form, $form_state) {
  $_SESSION['permission_watchdog_filter'] = array();
  foreach($form_state['values']['roles'] as $rid => $role) {
    if($role) {
      $_SESSION['permission_watchdog_filter']['roles'][$rid] = $rid;
    }
  }
}
